package com.huawei.demo;

import com.huawei.demo.api.OkHttpClientFactory;
import com.huawei.demo.api.TdpApi;
import com.huawei.demo.common.utils.EnvUtils;
import com.huawei.demo.model.AccessPointPara;
import com.huawei.demo.model.FunctionGraphPara;
import com.huawei.demo.model.TdpFuncBody;
import com.huawei.demo.model.TdpFuncBodyContext;
import com.huawei.demo.model.TdpFuncResult;
import com.huawei.demo.model.api.AllocateDeviceReq;
import com.huawei.demo.model.api.QueryDeviceBase;
import com.huawei.demo.model.error.ErrorEnum;
import com.huawei.services.runtime.Context;

import java.io.IOException;
import java.util.Optional;

import okhttp3.OkHttpClient;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.jackson.JacksonConverterFactory;

public class TdpFunctionImpl extends TdpFunction {

    /**
     * WARNING：该方法必须在子类中定义，否则函数无法触发该函数接口！！！
     */
    @Override
    public TdpFuncResult apiHandle(FunctionGraphPara para, Context context) {
        return super.apiHandle(para, context);
    }

    @Override
    protected TdpFuncResult checkPara(FunctionGraphPara para, Context context) {
        // 参数判空
        if (para == null) {
            LOGGER.warn("function input is null, return failed");
            return getFailResult(ErrorEnum.INVALID_FUNCTION_PARA);
        } else {
            LOGGER.info("input:{}", para.toString());
        }

        // 调试时使用，正式使用时建议删除
        EnvUtils.printContext(context);

        return null;
    }

    @Override
    protected AccessPointPara determineAccessPoint(FunctionGraphPara para) {
        return para.getAccessPoints() == null || para.getAccessPoints().size() == 0
            ? null : para.getAccessPoints().get(0);
    }

    @Override
    protected TdpFuncResult provisionDevice(FunctionGraphPara para, AccessPointPara accessPointPara) {
        if (accessPointPara == null) {
            LOGGER.warn("accessPointPara is null");
            return getFailResult(ErrorEnum.INVALID_ACCESS_POINT);
        }

        try {
            AllocateDeviceReq req = getAllocateDeviceReq(accessPointPara);
            String projectId = Optional.ofNullable(context).map(Context::getProjectID).orElse("");
            String xAuthToken = para.getIamToken();
            Response<QueryDeviceBase> response = retrofit().create(TdpApi.class)
                .allocateDevice(projectId, para.getDeviceId(), xAuthToken, req)
                .execute();

            return getResultFromResponse(response);
        } catch (Exception e) {
            return getFailResult(ErrorEnum.ERROR_IN_PROVISIONING_DEVICE);
        }
    }

    private AllocateDeviceReq getAllocateDeviceReq(AccessPointPara accessPointPara) {
        AllocateDeviceReq req = new AllocateDeviceReq();
        req.setRegionId(accessPointPara.getRegionId());
        req.setAccessPointId(accessPointPara.getAccessPointId());
        req.setAuthorizingDomainId(accessPointPara.getAuthorizingDomainId());
        return req;
    }

    private TdpFuncResult getResultFromResponse(Response<QueryDeviceBase> response) {
        if (response.isSuccessful()) {
            return getSuccessResult(Optional.ofNullable(response.body())
                .map(QueryDeviceBase::toString)
                .orElse(null));
        }

        String errorCode = ErrorEnum.ERROR_IN_PROVISIONING_DEVICE.getErrorCode();
        String errorMsg = Optional.ofNullable(response.errorBody())
            .map(body -> {
                try {
                    return body.string();
                } catch (IOException exp) {
                    return exp.getMessage();
                }
            })
            .orElse(ErrorEnum.ERROR_IN_PROVISIONING_DEVICE.getErrorDesc());
        return getFailResult(errorCode, errorMsg);
    }

    /**
     * 获取失败的响应结果
     *
     * @param errorCode 错误码
     * @param errorMsg  错误描述
     * @return 响应结果
     */
    private TdpFuncResult getFailResult(String errorCode, String errorMsg) {
        return new TdpFuncResult().withStatusCode(200)
            .withBody(new TdpFuncBody()
                .withStatus(TdpFuncBody.StatusEnum.DENY)
                .withContext(new TdpFuncBodyContext()
                    .withErrorCode(errorCode)
                    .withErrorMsg(errorMsg)
                )
            );
    }

    /**
     * 获取失败的响应结果
     *
     *  @return 响应结果
     */
    private TdpFuncResult getFailResult(ErrorEnum errorEnum) {
        return getFailResult(errorEnum.getErrorCode(), errorEnum.getErrorDesc());
    }

    /**
     * 获取成功的响应结果
     *
     * @param allocationResult 发放结果
     * @return 响应结果
     */
    private TdpFuncResult getSuccessResult(String allocationResult) {
        return new TdpFuncResult().withStatusCode(200)
            .withBody(new TdpFuncBody()
                .withStatus(TdpFuncBody.StatusEnum.ALLOW)
                .withContext(new TdpFuncBodyContext()
                    .withAllocationResult(allocationResult)
                )
            );
    }

    Retrofit retrofit() {
        OkHttpClient httpClient = OkHttpClientFactory.getOkHttpClient();

        return new Retrofit.Builder()
            .client(httpClient)
            .baseUrl("https://iotps.cn-north-4.myhuaweicloud.com")
            .addConverterFactory(JacksonConverterFactory.create())
            .build();
    }
}
