package com.huawei.demo.common.logger;

import com.huawei.services.runtime.RuntimeLogger;

import java.time.LocalDate;
import java.time.LocalTime;
import java.util.Arrays;
import java.util.stream.Collectors;

public class DefaultLogger implements RuntimeLogger, ExtendLogger {
    private volatile static RuntimeLogger INSTANCE = null;

    public static synchronized void init(RuntimeLogger logger) {
        INSTANCE = logger;
    }

    private Class clazz;

    public DefaultLogger(Class clazz) {
        this.clazz = clazz;
    }

    @Override
    public void log(String s) {
        if (INSTANCE != null) {
            INSTANCE.log(s);
            return;
        }

        if (s.contains("{ERROR}")) {
            System.err.println(String.format("%s %s:%s", LocalDate.now(), LocalTime.now(), s));
        } else {
            System.out.println(String.format("%s %s:%s", LocalDate.now(), LocalTime.now(), s));
        }
    }

    @Override
    public void debug(String msg) {
        log(appendLogLevel(LogLevel.DEBUG, msg));
    }

    @Override
    public void debug(String format, Object arg) {
        log(appendLogLevel(LogLevel.DEBUG, MessageFormatter.parse(format, arg)));
    }

    @Override
    public void debug(String format, Object arg1, Object arg2) {
        log(appendLogLevel(LogLevel.DEBUG, MessageFormatter.parse(format, arg1, arg2)));
    }

    @Override
    public void debug(String format, Object... arguments) {
        log(appendLogLevel(LogLevel.DEBUG, MessageFormatter.parse(format, arguments)));
    }

    @Override
    public void info(String msg) {
        log(appendLogLevel(LogLevel.INFO, msg));
    }

    @Override
    public void info(String format, Object arg) {
        log(appendLogLevel(LogLevel.INFO, MessageFormatter.parse(format, arg)));
    }

    @Override
    public void info(String format, Object arg1, Object arg2) {
        log(appendLogLevel(LogLevel.INFO, MessageFormatter.parse(format, arg1, arg2)));
    }

    @Override
    public void info(String format, Object... arguments) {
        log(appendLogLevel(LogLevel.INFO, MessageFormatter.parse(format, arguments)));
    }

    @Override
    public void warn(String msg) {
        log(appendLogLevel(LogLevel.WARN, msg));
    }

    @Override
    public void warn(String format, Object arg) {
        log(appendLogLevel(LogLevel.WARN, MessageFormatter.parse(format, arg)));
    }

    @Override
    public void warn(String format, Object arg1, Object arg2) {
        log(appendLogLevel(LogLevel.WARN, MessageFormatter.parse(format, arg1, arg2)));
    }

    @Override
    public void warn(String format, Object... arguments) {
        log(appendLogLevel(LogLevel.WARN, MessageFormatter.parse(format, arguments)));
    }


    @Override
    public void error(String msg) {
        log(appendLogLevel(LogLevel.ERROR, msg));
    }

    @Override
    public void error(String msg, Throwable throwable) {
        msg += Arrays.stream(throwable.getStackTrace())
                .map(trace -> String.format("\r\n\t\t%s.%s(%s:%d)",
                        trace.getClassName(),
                        trace.getMethodName(),
                        trace.getFileName(),
                        trace.getLineNumber()))
                .collect(Collectors.joining());
        log(appendLogLevel(LogLevel.ERROR, msg));
    }

    @Override
    public void error(String format, Object arg) {
        log(appendLogLevel(LogLevel.ERROR, MessageFormatter.parse(format, arg)));
    }

    @Override
    public void error(String format, Object arg1, Object arg2) {
        log(appendLogLevel(LogLevel.ERROR, MessageFormatter.parse(format, arg1, arg2)));
    }

    @Override
    public void error(String format, Object... arguments) {
        log(appendLogLevel(LogLevel.ERROR, MessageFormatter.parse(format, arguments)));
    }

    /**
     * [date time id] [LogLevel traceInfo: originMessage]
     * 2020/09/11 09:20:39 GMT+08:00  ca334771-378e-4b65-8f67-c9c46e6803bc {INFO} com.huawei.demo.logger.DefaultLogger.appendLogLevel(DefaultLogger.java:136)-->testContent
     */
    private String appendLogLevel(LogLevel logLevel, String originMessage) {
        String traceInfo = getTraceInfo();
        // com.huawei.demo.logger.DefaultLogger.appendLogLevel(DefaultLogger.java:136)-->
        return String.format("{%s} %s-->%s", logLevel.name(), traceInfo, originMessage);
    }

    public enum LogLevel {
        DEBUG, INFO, WARN, ERROR
    }

    private String getTraceInfo() {
        StackTraceElement[] trace = Thread.currentThread().getStackTrace();

        int methodCount = 1;
        int stackOffset = getStackOffset(trace, clazz);

        if (methodCount + stackOffset > trace.length) {
            methodCount = trace.length - stackOffset - 1;
        }

        for (int i = methodCount; i > 0; i--) {
            int stackIndex = i + stackOffset;
            if (stackIndex >= trace.length) {
                continue;
            }
            StackTraceElement element = trace[stackIndex];

            StringBuilder builder = new StringBuilder();
            builder.append(element.getClass().getSimpleName())
                    .append(".")
                    .append(element.getMethodName())
                    .append("(")
                    .append(element.getFileName())
                    .append(":")
                    .append(element.getLineNumber())
                    .append(")");

            return builder.toString();
        }

        return "";
    }

    private int getStackOffset(StackTraceElement[] trace, Class clazz) {
        for (int i = 2; i < trace.length; i++) {
            StackTraceElement e = trace[i];
            String name = e.getClassName();
            if (name.equals(clazz.getName())) {
                return --i;
            }
        }

        return -1;
    }
}
