package com.huawei.demo.model.api;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import java.io.Serializable;
import java.util.Objects;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

/**
 * 设备接入类型。
 **/
@JsonIgnoreProperties(ignoreUnknown = true)
@JsonSerialize(include = JsonSerialize.Inclusion.NON_NULL)
public class AuthInfo implements Serializable {
    private static final long serialVersionUID = 1L;

    @Pattern(regexp = "(SECRET|CERTIFICATES)")
    @Valid
    @JsonProperty("auth_type")
    private String authType = null;

    @Size(max = 32, min = 8)
    @Pattern(regexp = "^[a-zA-Z0-9_-]*$")
    @Valid
    @JsonProperty("secret")
    private String secret = null;

    @Pattern(regexp = "^[a-fA-F0-9]{40}$|^[a-fA-F0-9]{64}$")
    @Valid
    @JsonProperty("fingerprint")
    private String fingerprint = null;

    @Valid
    @JsonProperty("device_cert_name")
    private String deviceCertName = null;

    @Min(value = 1)
    @Max(value = 10)
    @Valid
    @JsonProperty("device_cert_validity")
    private Integer deviceCertValidity = null;

    @Valid
    @JsonProperty("device_cert_id")
    private String deviceCertId = null;

    @Valid
    @JsonProperty("secure_access")
    private Boolean secureAccess = true;

    @Valid
    @JsonProperty("timeout")
    private Integer timeout = 0;

    public AuthInfo() {
        super();
    }

    /**
     * 鉴权类型。支持秘钥认证接入(SECRET)和证书认证接入(CERTIFICATES)。使用秘钥认证接入方式(SECRET)填写secret字段，使用证书认证接入方式(CERTIFICATE)填写fingerprint字段，如果CA证书选择云证书管理服务的CA证书，不需要填写fingerprint字段，需要填写设备证书名称字段，不填写auth_type默认为秘钥认证接入方式(SECRET)
     * pattern: "(SECRET|CERTIFICATES)"
     **/
    public String getAuthType() {
        return authType;
    }

    public void setAuthType(String authType_in) {
        this.authType = authType_in;
    }

    /**
     * 设备秘钥，认证类型使用秘钥认证接入(SECRET)可填写该字段。注意：NB设备秘钥由于协议特殊性，只支持十六进制秘钥接入。
     * minLength: 8
     * maxLength: 32
     * pattern: "^[a-zA-Z0-9_-]*$"
     **/
    public String getSecret() {
        return secret;
    }

    public void setSecret(String secret_in) {
        this.secret = secret_in;
    }

    /**
     * 证书指纹，认证类型使用证书认证接入(CERTIFICATES)可填写该字段，注册设备时不填写该字段则取第一次设备接入时的证书指纹。注意：指纹只能为40位十六进制字符串或者64位十六进制字符串。
     * pattern: "^[a-fA-F0-9]{40}$|^[a-fA-F0-9]{64}$"
     **/
    public String getFingerprint() {
        return fingerprint;
    }

    public void setFingerprint(String fingerprint_in) {
        this.fingerprint = fingerprint_in;
    }

    /**
     * 设备证书名称，CA证书选择云证书管理服务的CA证书接入需要填写该字段。此名称会也即设备证书使用者的CommanName
     **/
    public String getDeviceCertName() {
        return deviceCertName;
    }

    public void setDeviceCertName(String deviceCertName_in) {
        this.deviceCertName = deviceCertName_in;
    }

    /**
     * 生成设备证书的有效期。CA证书选择云证书管理服务的CA证书可填写该字段。
     * minimum: 1.0
     * maximum: 10.0
     **/
    public Integer getDeviceCertValidity() {
        return deviceCertValidity;
    }

    public void setDeviceCertValidity(Integer deviceCertValidity_in) {
        this.deviceCertValidity = deviceCertValidity_in;
    }

    /**
     * 设备证书id，设备采用云证书管理服务的CA注册，云证书管理服务生成的设备证书的唯一实例ID
     **/
    public String getDeviceCertId() {
        return deviceCertId;
    }

    public void setDeviceCertId(String deviceCertId_in) {
        this.deviceCertId = deviceCertId_in;
    }

    /**
     * 指设备是否通过安全协议方式接入，默认值为true。\n- true：通过安全协议方式接入。\n- false：通过非安全协议方式接入。
     **/
    public Boolean getSecureAccess() {
        return secureAccess;
    }

    public void setSecureAccess(Boolean secureAccess_in) {
        this.secureAccess = secureAccess_in;
    }

    /**
     * 设备验证码的有效时间，单位：秒，默认值：0\n若设备在有效时间内未接入物联网平台并激活，则平台会删除该设备的注册信息。若设置为“0”，则表示设备验证码不会失效（建议填写为“0”）。
     **/
    public Integer getTimeout() {
        return timeout;
    }

    public void setTimeout(Integer timeout_in) {
        this.timeout = timeout_in;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        AuthInfo authInfo = (AuthInfo) o;
        return Objects.equals(this.authType, authInfo.authType)
            && Objects.equals(this.secret, authInfo.secret)
            && Objects.equals(this.fingerprint, authInfo.fingerprint)
            && Objects.equals(this.deviceCertName, authInfo.deviceCertName)
            && Objects.equals(this.deviceCertValidity, authInfo.deviceCertValidity)
            && Objects.equals(this.deviceCertId, authInfo.deviceCertId)
            && Objects.equals(this.secureAccess, authInfo.secureAccess)
            && Objects.equals(this.timeout, authInfo.timeout);
    }

    @Override
    public int hashCode() {
        return Objects.hash(authType, secret, fingerprint, deviceCertName, deviceCertValidity, deviceCertId,
            secureAccess, timeout);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("class AuthInfo { ");

        sb.append("  authType: ").append(authType).append(", ");
        sb.append("  secret: ").append(secret).append(", ");
        sb.append("  fingerprint: ").append(fingerprint).append(", ");
        sb.append("  deviceCertName: ").append(deviceCertName).append(", ");
        sb.append("  deviceCertValidity: ").append(deviceCertValidity).append(", ");
        sb.append("  deviceCertId: ").append(deviceCertId).append(", ");
        sb.append("  secureAccess: ").append(secureAccess).append(", ");
        sb.append("  timeout: ").append(timeout).append(", ");
        sb.append("} ");
        return sb.toString();
    }
}
