package com.huawei.demo.model.api;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import java.io.Serializable;
import java.util.Objects;

import javax.validation.Valid;
import javax.validation.constraints.Pattern;
import javax.validation.constraints.Size;

/**
 * 设备信息结构体，返回基本信息。
 **/
@JsonIgnoreProperties(ignoreUnknown = true)
@JsonSerialize(include = JsonSerialize.Inclusion.NON_NULL)
public class QueryDeviceBase implements Serializable {
    private static final long serialVersionUID = 1L;

    @Size(max = 256, min = 0)
    @Valid
    @JsonProperty("device_id")
    private String deviceId = null;

    @Size(max = 64, min = 0)
    @Valid
    @JsonProperty("node_id")
    private String nodeId = null;

    @Size(max = 256, min = 0)
    @Valid
    @JsonProperty("gateway_id")
    private String gatewayId = null;

    @Size(max = 256, min = 0)
    @Valid
    @JsonProperty("device_name")
    private String deviceName = null;

    @Pattern(regexp = "(ENDPOINT|GATEWAY|UNKNOWN)")
    @Valid
    @JsonProperty("node_type")
    private String nodeType = null;

    @Size(max = 2048, min = 0)
    @Valid
    @JsonProperty("description")
    private String description = null;

    @Valid
    @JsonProperty("auth_info")
    private AuthInfo authInfo = null;

    @Pattern(regexp = "^[a-f0-9-]{1,36}$")
    @Valid
    @JsonProperty("product_id")
    private String productId = null;

    @Size(max = 255, min = 1)
    @Valid
    @JsonProperty("product_name")
    private String productName = null;

    @Pattern(regexp = "(MQTT|LWM2M)")
    @Valid
    @JsonProperty("protocol_type")
    private String protocolType = null;

    @Size(max = 128, min = 0)
    @Valid
    @JsonProperty("create_time")
    private String createTime = null;

    @Valid
    @JsonProperty("provision_info")
    private ProvisionInfoBaseResp provisionInfo = null;

    public QueryDeviceBase() {
        super();
    }

    /**
     * 设备ID，用于唯一标识一个设备，在注册设备时由物联网平台分配获得。（生成规则为\"product_id\" + \"_\" + \"node_id\"拼接而成）
     * minLength: 0
     * maxLength: 256
     **/
    public String getDeviceId() {
        return deviceId;
    }

    public void setDeviceId(String deviceId_in) {
        this.deviceId = deviceId_in;
    }

    /**
     * 设备标识码，通常使用IMEI、MAC地址或Serial No作为nodeId。
     * minLength: 0
     * maxLength: 64
     **/
    public String getNodeId() {
        return nodeId;
    }

    public void setNodeId(String nodeId_in) {
        this.nodeId = nodeId_in;
    }

    /**
     * 网关ID，用于标识一个网关设备。当设备是直连设备时，gatewayId与设备的deviceId一致。当设备是非直连设备时，gatewayId为设备所关联的直连设备（即网关）的deviceId。
     * minLength: 0
     * maxLength: 256
     **/
    public String getGatewayId() {
        return gatewayId;
    }

    public void setGatewayId(String gatewayId_in) {
        this.gatewayId = gatewayId_in;
    }

    /**
     * 设备名称。
     * minLength: 0
     * maxLength: 256
     **/
    public String getDeviceName() {
        return deviceName;
    }

    public void setDeviceName(String deviceName_in) {
        this.deviceName = deviceName_in;
    }

    /**
     * 设备节点类型。\n- ENDPOINT：非直连设备。\n- GATEWAY：直连设备或网关。\n- UNKNOWN：未知。
     * pattern: "(ENDPOINT|GATEWAY|UNKNOWN)"
     **/
    public String getNodeType() {
        return nodeType;
    }

    public void setNodeType(String nodeType_in) {
        this.nodeType = nodeType_in;
    }

    /**
     * 设备的描述信息。
     * minLength: 0
     * maxLength: 2048
     **/
    public String getDescription() {
        return description;
    }

    public void setDescription(String description_in) {
        this.description = description_in;
    }

    /**
     * 设备的接入认证信息。
     **/
    public AuthInfo getAuthInfo() {
        return authInfo;
    }

    public void setAuthInfo(AuthInfo authInfo_in) {
        this.authInfo = authInfo_in;
    }

    /**
     * 设备关联的产品ID，用于唯一标识一个产品模型。
     * pattern: "^[a-f0-9-]{1,36}$"
     **/
    public String getProductId() {
        return productId;
    }

    public void setProductId(String productId_in) {
        this.productId = productId_in;
    }

    /**
     * 设备关联的产品名称。
     * minLength: 1
     * maxLength: 255
     **/
    public String getProductName() {
        return productName;
    }

    public void setProductName(String productName_in) {
        this.productName = productName_in;
    }

    /**
     * 设备关联的产品协议类型。
     * pattern: "(MQTT|LWM2M)"
     **/
    public String getProtocolType() {
        return protocolType;
    }

    public void setProtocolType(String protocolType_in) {
        this.protocolType = protocolType_in;
    }

    /**
     * 在物联网平台注册设备的时间。
     * minLength: 0
     * maxLength: 128
     **/
    public String getCreateTime() {
        return createTime;
    }

    public void setCreateTime(String createTime_in) {
        this.createTime = createTime_in;
    }

    /**
     * 设备的发放信息。
     **/
    public ProvisionInfoBaseResp getProvisionInfo() {
        return provisionInfo;
    }

    public void setProvisionInfo(ProvisionInfoBaseResp provisionInfo_in) {
        this.provisionInfo = provisionInfo_in;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        QueryDeviceBase queryDeviceBase = (QueryDeviceBase) o;
        return Objects.equals(this.deviceId, queryDeviceBase.deviceId)
            && Objects.equals(this.nodeId, queryDeviceBase.nodeId)
            && Objects.equals(this.gatewayId, queryDeviceBase.gatewayId)
            && Objects.equals(this.deviceName, queryDeviceBase.deviceName)
            && Objects.equals(this.nodeType, queryDeviceBase.nodeType)
            && Objects.equals(this.description, queryDeviceBase.description)
            && Objects.equals(this.authInfo, queryDeviceBase.authInfo)
            && Objects.equals(this.productId, queryDeviceBase.productId)
            && Objects.equals(this.productName, queryDeviceBase.productName)
            && Objects.equals(this.protocolType, queryDeviceBase.protocolType)
            && Objects.equals(this.createTime, queryDeviceBase.createTime)
            && Objects.equals(this.provisionInfo, queryDeviceBase.provisionInfo);
    }

    @Override
    public int hashCode() {
        return Objects.hash(deviceId, nodeId, gatewayId, deviceName, nodeType, description, authInfo, productId,
            productName, protocolType, createTime, provisionInfo);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("class QueryDeviceBase { ");

        sb.append("  deviceId: ").append(deviceId).append(", ");
        sb.append("  nodeId: ").append(nodeId).append(", ");
        sb.append("  gatewayId: ").append(gatewayId).append(", ");
        sb.append("  deviceName: ").append(deviceName).append(", ");
        sb.append("  nodeType: ").append(nodeType).append(", ");
        sb.append("  description: ").append(description).append(", ");
        sb.append("  authInfo: ").append(authInfo).append(", ");
        sb.append("  productId: ").append(productId).append(", ");
        sb.append("  productName: ").append(productName).append(", ");
        sb.append("  protocolType: ").append(protocolType).append(", ");
        sb.append("  createTime: ").append(createTime).append(", ");
        sb.append("  provisionInfo: ").append(provisionInfo).append(", ");
        sb.append("} ");
        return sb.toString();
    }
}
